/* wydawca - automatic release submission daemon
   Copyright (C) 2007, 2009 Sergey Poznyakoff

   Wydawca is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3 of the License, or (at your
   option) any later version.

   Wydawca is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with wydawca. If not, see <http://www.gnu.org/licenses/>. */

#include "wydawca.h"

struct spool_list
{
  struct spool_list *next;
  struct spool spool;
};

static struct spool_list *spool_list;

void
register_spool (struct spool *spool)
{
  struct spool_list *sp = xmalloc (sizeof *sp);
  sp->spool = *spool;
  sp->next = spool_list;
  spool_list = sp;
}

/* Return true if NAME is a directory. If stat fails, return the error
   code in EC */
int
test_dir (const char *name, int *ec)
{
  struct stat st;

  *ec = 0;
  if (stat (name, &st))
    {
      *ec = errno;
      return 1;
    }
  return S_ISDIR (st.st_mode) == 0;
}

/* Return a textual representation of a file TYPE */
const char *
file_type_str (enum file_type type)
{
  switch (type)
    {
    case file_dist:
      return "distributive";

    case file_signature:
      return "detached signature";

    case file_directive:
      return "signed upload directive";
    }
  return "UNKNOWN";
}

/* Parse file NAME: determine its type and root name and store this
   information in FINFO */
void
parse_file_name (const char *name, struct file_info *finfo)
{
  static struct suffix
  {
    const char *suf;
    unsigned len;
    enum file_type type;
  } suftab[] = {
    { SUF_SIG, SUF_SIG_LEN, file_signature },
    { SUF_DIR, SUF_DIR_LEN, file_directive },
    { "", 0, file_dist }
  };
  int i;
  unsigned len = strlen (name);

  for (i = 0; i < sizeof suftab / sizeof suftab[0]; i++)
    {
      if (len >= suftab[i].len
	  && memcmp (name + len - suftab[i].len,
		     suftab[i].suf, suftab[i].len) == 0)
	{
	  finfo->name = xstrdup (name);
	  finfo->type = suftab[i].type;
	  finfo->root_len = len - suftab[i].len;
	  return;
	}
    }
  abort (); /* should not happen */
}

/* Scan upload directory from the DPAIR and register all files found
   there, forming triplets when possible */
void
scan_spool (struct spool *spool)
{
  DIR *dir;
  struct dirent *ent;

  if (debug_level)
    logmsg (LOG_DEBUG, "%s -> %s", spool->source_dir,
	    mu_url_to_string (spool->dest_url));

  if (chdir (spool->source_dir))
    {
      logmsg (LOG_ERR, "cannot chdir to %s: %s", spool->source_dir,
	      strerror (errno));
      return;
    }

  dir = opendir (".");
  if (!dir)
    {
      logmsg (LOG_ERR, "cannot open directory %s: %s", spool->source_dir,
	      strerror (errno));
      return;
    }

  timer_start ("directory");
  timer_start (spool->url);
  while ((ent = readdir (dir)))
    {
      struct stat st;
      struct file_info finfo;

      if (strcmp (ent->d_name, ".") == 0 || strcmp (ent->d_name, "..") == 0)
	continue;

      if (stat (ent->d_name, &st))
	{
	  logmsg (LOG_ERR, "cannot stat file %s/%s: %s",
		  spool->source_dir, ent->d_name,
		  strerror (errno));
	  continue;
	}

      if (!S_ISREG (st.st_mode))
	{
	  logmsg (LOG_NOTICE, "not a regular file: %s/%s",
		  spool->source_dir, ent->d_name);
	  continue;
	}

      if (!match_uid_p (st.st_uid))
	{
	  if (debug_level)
	    logmsg (LOG_DEBUG, "ignoring file: %s/%s",
		    spool->source_dir, ent->d_name);
	  continue;
	}
      
      finfo.sb = st;
      parse_file_name (ent->d_name, &finfo);

      if (debug_level)
	{
	  const char *s = file_type_str (finfo.type);
	  logmsg (LOG_DEBUG, "file %s is %s %s, root %.*s", ent->d_name,
		  strchr ("aeiou", s[0]) ? "an" : "a", s,
		  finfo.root_len, finfo.name);
	}

      register_file (&finfo);
    }

  closedir (dir);

  if (count_collected_triplets () > 0)
    {
      int i;

      for (i = 0; i < access_method_count; i++)
        {
          if (method_init (spool->access_method[i]))
            {
              logmsg (LOG_ERR, "failed to initialize access method %d", i);
              return;
            }
        }
      enumerate_triplets (spool);
   }
  timer_stop (spool->url);
  timer_stop ("directory");
}

static void
close_methods (struct spool *spool)
{
  int i;
  for (i = 0; i < NITEMS (spool->access_method); i++)
    method_done (spool->access_method[i]);
}

/* Scan all configured update directories */
void
scan_directories ()
{
  struct spool_list *sp;

  timer_start ("wydawca");

  for (sp = spool_list; sp; sp = sp->next)
    if (enabled_spool_p (&sp->spool))
      scan_spool (&sp->spool);

  for (sp = spool_list; sp; sp = sp->next)
    close_methods (&sp->spool);
  timer_stop ("wydawca");
}

